@@ -1,6 +1,6 @@
 /*
  * This file is part of the KDE Baloo Project
- * Copyright (C) 2013  Vishesh Handa <me@vhanda.in>
+ * Copyright (C) 2013-2015  Vishesh Handa <vhanda@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,6 @@
 
 #include "result.h"
 
-#include <QDebug>
 #include <QJsonDocument>
 #include <QJsonObject>
 
@@ -34,23 +33,24 @@
 Result::Result()
     : ExtractionResult(QString(), QString())
     , m_docId(0)
-    , m_termGen(0)
-    , m_termGenForText(0)
+    , m_termGen(nullptr)
+    , m_termGenForText(nullptr)
 {
 }
 
 Result::Result(const QString& url, const QString& mimetype, const Flags& flags)
     : KFileMetaData::ExtractionResult(url, mimetype, flags)
     , m_docId(0)
-    , m_termGen(0)
-    , m_termGenForText(0)
+    , m_termGen(nullptr)
+    , m_termGenForText(nullptr)
 {
 }
 
 void Result::add(KFileMetaData::Property::Property property, const QVariant& value)
 {
-    QString p = QString::number(static_cast<int>(property));
+    int propNum = static_cast<int>(property);
     if (!value.isNull()) {
+        QString p = QString::number(propNum);
         if (!m_map.contains(p)) {
             m_map.insert(p, value);
         } else {
@@ -67,22 +67,35 @@ void Result::add(KFileMetaData::Property::Property property, const QVariant& val
         }
     }
 
-    QString prefix = QLatin1Char('X') + p;
+    QByteArray prefix = 'X' + QByteArray::number(propNum) + '-';
 
     if (value.type() == QVariant::Bool) {
-        m_doc.add_boolean_term(prefix.toUtf8().constData());
+        m_doc.addBoolTerm(prefix);
     }
     else if (value.type() == QVariant::Int) {
-        const QString term = prefix + value.toString();
-        m_doc.add_term(term.toUtf8().constData());
+        const QByteArray term = prefix + value.toString().toUtf8();
+        m_doc.addBoolTerm(term);
     }
     else if (value.type() == QVariant::Date) {
-        const QString term = prefix + value.toDate().toString(Qt::ISODate);
-        m_doc.add_term(term.toUtf8().constData());
+        const QByteArray term = prefix + value.toDate().toString(Qt::ISODate).toUtf8();
+        m_doc.addBoolTerm(term);
     }
     else if (value.type() == QVariant::DateTime) {
-        const QString term = prefix + value.toDateTime().toString(Qt::ISODate);
-        m_doc.add_term(term.toUtf8().constData());
+        const QByteArray term = prefix + value.toDateTime().toString(Qt::ISODate).toUtf8();
+        m_doc.addBoolTerm(term);
+    }
+    else if (value.type() == QVariant::StringList) {
+        bool shouldBeIndexed = KFileMetaData::PropertyInfo(property).shouldBeIndexed();
+        const auto valueList = value.toStringList();
+        for (const auto& val : valueList)
+        {
+            if (val.isEmpty())
+                continue;
+            m_termGen.indexText(val, prefix);
+            if (shouldBeIndexed)
+                m_termGen.indexText(val);
+
+        }
     }
     else {
         const QString val = value.toString();
@@ -105,7 +118,7 @@ void Result::addType(KFileMetaData::Type::Type type)
 {
     KFileMetaData::TypeInfo ti(type);
     const QString t = QLatin1Char('T') + ti.name().toLower();
-    m_doc.add_boolean_term(t.toUtf8().constData());
+    m_doc.addBoolTerm(t.toUtf8());
 }
 
 void Result::finish()
@@ -113,10 +126,10 @@ void Result::finish()
     QJsonObject jo = QJsonObject::fromVariantMap(m_map);
     QJsonDocument jdoc;
     jdoc.setObject(jo);
-    m_doc.set_data(jdoc.toJson().constData());
+    m_doc.setData(jdoc.toJson());
 }
 
-void Result::setDocument(const Xapian::Document& doc)
+void Result::setDocument(const Baloo::Document& doc)
 {
     m_doc = doc;
     // All document metadata are indexed from position 1000
@@ -129,12 +142,7 @@ void Result::setDocument(const Xapian::Document& doc)
     m_termGenForText.setPosition(10000);
 }
 
-void Result::setId(uint id)
-{
-    m_docId = id;
-}
-
-uint Result::id() const
+quint64 Result::id() const
 {
     return m_docId;
 }
